﻿using Autodesk.Forge;
using Autodesk.Forge.Model;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Threading.Tasks;
using System.Web;
using System.Web.Http;

namespace Trabajo_Buckets.Controllers
{
    public class OSSController : ApiController
    {
        public string ClientId { get { return OAuthController.GetAppSetting("FORGE_CLIENT_ID").ToLower(); } }

        /// <summary>
        /// Return list of buckets (id=#) or list of objects (id=bucketKey)
        /// </summary>
        [HttpGet]
        [Route("api/forge/oss/buckets")]
        public async Task<IList<TreeNode>> GetOSSAsync([FromUri] string id)
        {
            IList<TreeNode> nodes = new List<TreeNode>();
            dynamic oauth = await OAuthController.GetInternalAsync();

            if (id == "#") // root
            {
                // in this case, let's return all buckets
                BucketsApi appBckets = new BucketsApi();
                appBckets.Configuration.AccessToken = oauth.access_token;

                // to simplify, let's return only the first 100 buckets
                //dynamic buckets = await appBckets.GetBucketsAsync("US",3);
                //foreach (KeyValuePair<string, dynamic> bucket in new DynamicDictionaryItems(buckets.items))
                //{
                //    nodes.Add(new TreeNode(bucket.Value.bucketKey, bucket.Value.bucketKey.Replace(ClientId + "-", string.Empty), "bucket", true));
                //}
                bool continuar = true;
                string mBuketKey = null;
                while (continuar == true)
                {
                    dynamic buckets = appBckets.GetBuckets("US", 5, mBuketKey);
                    //creamos una nueva variable de tipo DynamicDictionaryItems
                    var keyValuePairs = new DynamicDictionaryItems(buckets.items);
                    if (keyValuePairs.Count() == 0)
                    {
                        continuar = false;
                        break;
                    }
                    foreach (KeyValuePair<string, dynamic> bucket in keyValuePairs)
                    {
                        nodes.Add(new TreeNode(bucket.Value.bucketKey, bucket.Value.bucketKey.Replace
                        (ClientId + "-", string.Empty), "bucket", true));
                        //Guardamos en mBuketKey el valor. aunque solo utilizaremos el último
                        mBuketKey = bucket.Value.bucketKey;
                    }
                }
            }
            else
            {
                // as we have the id (bucketKey), let's return all 
                ObjectsApi objects = new ObjectsApi();
                objects.Configuration.AccessToken = oauth.access_token;
                //var objectsList = objects.GetObjects(id);
                //foreach (KeyValuePair<string, dynamic> objInfo in new DynamicDictionaryItems(objectsList.items))
                //{
                //    nodes.Add(new TreeNode(Base64Encode((string)objInfo.Value.objectId),
                //      objInfo.Value.objectKey, "object", false));
                //}
                string startAt = null;
                bool continuar = true;
                while (continuar == true)
                {
                    var objectsList = objects.GetObjects(id, 5, null, startAt);
                    if (objectsList.Count == 2)
                        startAt = (objectsList.next).Split('=')[1].Split('&')[0];
                    else continuar = false;
                    startAt = HttpUtility.UrlDecode(startAt);
                    var keyValuePairs = new DynamicDictionaryItems(objectsList.items);
                    foreach (KeyValuePair<string, dynamic> objInfo in keyValuePairs)
                    {
                        nodes.Add(new TreeNode(Base64Encode((string)objInfo.Value.objectId),
                        objInfo.Value.objectKey, "object", false));
                    }
                }
            }
            return nodes;
        }

        /// <summary>
        /// Model data for jsTree used on GetOSSAsync
        /// </summary>
        public class TreeNode
        {
            public TreeNode(string id, string text, string type, bool children)
            {
                this.id = id;
                this.text = text;
                this.type = type;
                this.children = children;
            }

            public string id { get; set; }
            public string text { get; set; }
            public string type { get; set; }
            public bool children { get; set; }
        }

        /// <summary>
        /// Create a new bucket 
        /// </summary>
        [HttpPost]
        [Route("api/forge/oss/buckets")]
        public async Task<dynamic> CreateBucket([FromBody] CreateBucketModel bucket)
        {
            BucketsApi buckets = new BucketsApi();
            dynamic token = await OAuthController.GetInternalAsync();
            buckets.Configuration.AccessToken = token.access_token;
            PostBucketsPayload bucketPayload = new PostBucketsPayload(string.Format("{0}-{1}", ClientId, bucket.bucketKey.ToLower()), null,
              PostBucketsPayload.PolicyKeyEnum.Transient);
            return await buckets.CreateBucketAsync(bucketPayload, "US");
        }

        /// <summary>
        /// Delete a bucket 
        /// </summary>
        [HttpPost]
        [Route("api/forge/oss/bucketsDelete")]
        public async Task<dynamic> DeleteBucket([FromBody] CreateBucketModel bucket)
        {
            BucketsApi buckets = new BucketsApi();
            dynamic token = await OAuthController.GetInternalAsync();
            buckets.Configuration.AccessToken = token.access_token;
            await buckets.DeleteBucketAsync(bucket.bucketKey.ToLower());
            return null;
        }

        /// <summary>
        /// Delete a object 
        /// </summary>
        [HttpPost]
        [Route("api/forge/oss/objects/delete")]
        public async Task<dynamic> DeleteObjects([FromBody] DeleteObjectModel objInfo)
        {
            ObjectsApi objects = new ObjectsApi();
            dynamic token = await OAuthController.GetInternalAsync();
            objects.Configuration.AccessToken = token.access_token;
            await objects.DeleteObjectAsync(objInfo.bucketKey.ToLower(), objInfo.objectKey);
            return null;
        }

        /// <summary>
        /// Input model for DeleteObjects method
        /// </summary>
        public class DeleteObjectModel
        {
            public string bucketKey { get; set; }
            public string objectKey { get; set; }
        }
        /// <summary>
        /// Input model for CreateBucket method
        /// </summary>
        public class CreateBucketModel
        {
            public string bucketKey { get; set; }
        }

        /// <summary>
        /// Receive a file from the client and upload to the bucket
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        [Route("api/forge/oss/objects")]
        public async Task<dynamic> UploadObject()
        {
            // basic input validation
            HttpRequest req = HttpContext.Current.Request;
            if (string.IsNullOrWhiteSpace(req.Params["bucketKey"]))
                throw new System.Exception("BucketKey parameter was not provided.");

            if (req.Files.Count != 1)
                throw new System.Exception("Missing file to upload"); // for now, let's support just 1 file at a time

            string bucketKey = req.Params["bucketKey"];
            HttpPostedFile file = req.Files[0];

            // save the file on the server
            var fileSavePath = Path.Combine(HttpContext.Current.Server.MapPath("~/App_Data"), file.FileName);
            file.SaveAs(fileSavePath);

            // get the bucket...
            dynamic oauth = await OAuthController.GetInternalAsync();
            ObjectsApi objects = new ObjectsApi();
            objects.Configuration.AccessToken = oauth.access_token;

            // upload the file/object, which will create a new object
            dynamic uploadedObj;
            using (StreamReader streamReader = new StreamReader(fileSavePath))
            {
                uploadedObj = await objects.UploadObjectAsync(bucketKey,
                       file.FileName, (int)streamReader.BaseStream.Length, streamReader.BaseStream,
                       "application/octet-stream");
            }

            // cleanup
            File.Delete(fileSavePath);

            return uploadedObj;
        }

        /// <summary>
        /// Base64 enconde a string
        /// </summary>
        public static string Base64Encode(string plainText)
        {
            var plainTextBytes = System.Text.Encoding.UTF8.GetBytes(plainText);
            return System.Convert.ToBase64String(plainTextBytes);
        }
    }
}
