/* 
 * Forge SDK
 *
 * The Forge Platform contains an expanding collection of web service components that can be used with Autodesk cloud-based products or your own technologies. Take advantage of Autodesk’s expertise in design and engineering.
 *
 * OpenAPI spec version: 0.1.0
 * Contact: forge.help@autodesk.com
 * Generated by: https://github.com/swagger-api/swagger-codegen.git
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using RestSharp;
using Newtonsoft.Json.Linq;
using Autodesk.Forge.Client;
using Autodesk.Forge.Model;
using Version=Autodesk.Forge.Model.Version;

namespace Autodesk.Forge
{
    /// <summary>
    /// Represents a collection of functions to interact with the API endpoints
    /// </summary>
    public interface IItemsApi : IApiAccessor
    {
        #region Synchronous Operations
        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns a resource item by ID for any item within a given project. Resource items represent word documents, fusion design files, drawings, spreadsheets, etc. 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <returns>Item</returns>
        /*Item*/dynamic GetItem (string projectId, string itemId);

        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns a resource item by ID for any item within a given project. Resource items represent word documents, fusion design files, drawings, spreadsheets, etc. 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <returns>ApiResponse of Item</returns>
        ApiResponse</*Item*/dynamic> GetItemWithHttpInfo (string projectId, string itemId);
        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns the \&quot;parent\&quot; folder for the given item. 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <returns>Folder</returns>
        /*Folder*/dynamic GetItemParentFolder (string projectId, string itemId);

        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns the \&quot;parent\&quot; folder for the given item. 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <returns>ApiResponse of Folder</returns>
        ApiResponse</*Folder*/dynamic> GetItemParentFolderWithHttpInfo (string projectId, string itemId);

        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns the resources (&#x60;items&#x60;, &#x60;folders&#x60;, and &#x60;versions&#x60;) which have a custom relationship with the given &#x60;item_id&#x60;. Custom relationships can be established between an item and other resources within the &#39;data&#39; domain service (folders, items, and versions). 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <returns>JsonApiCollection</returns>
        /*JsonApiCollection*/dynamic GetItemRefs (string projectId, string itemId, List<string> filterType = null, List<string> filterId = null, List<string> filterExtensionType = null);

        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns the resources (&#x60;items&#x60;, &#x60;folders&#x60;, and &#x60;versions&#x60;) which have a custom relationship with the given &#x60;item_id&#x60;. Custom relationships can be established between an item and other resources within the &#39;data&#39; domain service (folders, items, and versions). 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <returns>ApiResponse of JsonApiCollection</returns>
        ApiResponse</*JsonApiCollection*/dynamic> GetItemRefsWithHttpInfo (string projectId, string itemId, List<string> filterType = null, List<string> filterId = null, List<string> filterExtensionType = null);

        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns a collection of links for the given item_id. Custom relationships can be established between an item and other external resources residing outside the data domain service. A link’s href defines the target URI to access a resource.
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterMimeType">Filter by mime type. (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <returns>Links</returns>
        /*Links*/
        dynamic GetItemRelationshipsLinks (string projectId, string itemId, List<string> filterType = null, List<string> filterId = null, List<string> filterMimeType = null, List<string> filterExtensionType = null);

        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns a collection of links for the given item_id. Custom relationships can be established between an item and other external resources residing outside the data domain service. A link’s href defines the target URI to access a resource.
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterMimeType">Filter by mime type. (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <returns>ApiResponse of Links</returns>
        ApiResponse</*Refs*/dynamic> GetItemRelationshipsLinksWithHttpInfo (string projectId, string itemId, List<string> filterType = null, List<string> filterId = null, List<string> filterMimeType = null, List<string> filterExtensionType = null);

        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns the custom relationships that are associated to the given &#x60;item_id&#x60;. Custom relationships can be established between an item and other resources within the &#39;data&#39; domain service (folders, items, and versions). 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterRefType">filter by &#x60;refType&#x60; (optional)</param>
        /// <param name="filterDirection">filter by the direction of the reference (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <returns>Refs</returns>
        /*Refs*/
        dynamic GetItemRelationshipsRefs (string projectId, string itemId, List<string> filterType = null, List<string> filterId = null, List<string> filterRefType = null, string filterDirection = null, List<string> filterExtensionType = null);

        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns the custom relationships that are associated to the given &#x60;item_id&#x60;. Custom relationships can be established between an item and other resources within the &#39;data&#39; domain service (folders, items, and versions). 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterRefType">filter by &#x60;refType&#x60; (optional)</param>
        /// <param name="filterDirection">filter by the direction of the reference (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <returns>ApiResponse of Refs</returns>
        ApiResponse</*Refs*/dynamic> GetItemRelationshipsRefsWithHttpInfo (string projectId, string itemId, List<string> filterType = null, List<string> filterId = null, List<string> filterRefType = null, string filterDirection = null, List<string> filterExtensionType = null);

        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns the \&quot;tip\&quot; version for the given item. Multiple versions of a resource item can be uploaded in a project. The tip version is the most recent one. 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <returns>Version</returns>
        /*Version*/dynamic GetItemTip (string projectId, string itemId);

        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns the \&quot;tip\&quot; version for the given item. Multiple versions of a resource item can be uploaded in a project. The tip version is the most recent one. 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <returns>ApiResponse of Version</returns>
        ApiResponse</*Version*/dynamic> GetItemTipWithHttpInfo (string projectId, string itemId);
        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns versions for the given item. Multiple versions of a resource item can be uploaded in a project. 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <param name="filterVersionNumber">filter by &#x60;versionNumber&#x60; (optional)</param>
        /// <param name="pageNumber">specify the page number (optional)</param>
        /// <param name="pageLimit">specify the maximal number of elements per page (optional)</param>
        /// <returns>Versions</returns>
        /*Versions*/dynamic GetItemVersions (string projectId, string itemId, List<string> filterType = null, List<string> filterId = null, List<string> filterExtensionType = null, List<int?> filterVersionNumber = null, int? pageNumber = null, int? pageLimit = null);

        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns versions for the given item. Multiple versions of a resource item can be uploaded in a project. 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <param name="filterVersionNumber">filter by &#x60;versionNumber&#x60; (optional)</param>
        /// <param name="pageNumber">specify the page number (optional)</param>
        /// <param name="pageLimit">specify the maximal number of elements per page (optional)</param>
        /// <returns>ApiResponse of Versions</returns>
        ApiResponse</*Versions*/dynamic> GetItemVersionsWithHttpInfo (string projectId, string itemId, List<string> filterType = null, List<string> filterId = null, List<string> filterExtensionType = null, List<int?> filterVersionNumber = null, int? pageNumber = null, int? pageLimit = null);
        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Creates a new item in the &#39;data&#39; domain service. 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="body">describe the item to be created</param>
        /// <returns>ItemCreated</returns>
        /*ItemCreated*/dynamic PostItem (string projectId, CreateItem body);

        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Creates a new item in the &#39;data&#39; domain service. 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="body">describe the item to be created</param>
        /// <returns>ApiResponse of ItemCreated</returns>
        ApiResponse</*ItemCreated*/dynamic> PostItemWithHttpInfo (string projectId, CreateItem body);
        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Creates a custom relationship between an item and another resource within the &#39;data&#39; domain service (folder, item, or version). 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <param name="body">describe the ref to be created</param>
        /// <returns></returns>
        dynamic PostItemRelationshipsRef (string projectId, string itemId, CreateRef body);

        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Creates a custom relationship between an item and another resource within the &#39;data&#39; domain service (folder, item, or version). 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <param name="body">describe the ref to be created</param>
        /// <returns>ApiResponse of Object(void)</returns>
        ApiResponse<dynamic> PostItemRelationshipsRefWithHttpInfo (string projectId, string itemId, CreateRef body);

        /// <summary>
        ///  Updates the properties of the given item_id object.
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <param name="body">describe the ref to be created</param>
        /// <returns></returns>
        dynamic PatchItem (string projectId, string itemId, PatchItem body);

        /// <summary>
        /// Updates the properties of the given item_id object
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <param name="body">describe the ref to be created</param>
        /// <returns>ApiResponse</returns>
        ApiResponse<dynamic> PatchItemWithHttpInfo (string projectId, string itemId, PatchItem body);

        #endregion Synchronous Operations
        #region Asynchronous Operations
        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns a resource item by ID for any item within a given project. Resource items represent word documents, fusion design files, drawings, spreadsheets, etc. 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <returns>Task of Item</returns>
        System.Threading.Tasks.Task</*Item*/dynamic> GetItemAsync (string projectId, string itemId);

        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns a resource item by ID for any item within a given project. Resource items represent word documents, fusion design files, drawings, spreadsheets, etc. 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <returns>Task of ApiResponse (Item)</returns>
        System.Threading.Tasks.Task<ApiResponse</*Item*/dynamic>> GetItemAsyncWithHttpInfo (string projectId, string itemId);
        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns the \&quot;parent\&quot; folder for the given item. 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <returns>Task of Folder</returns>
        System.Threading.Tasks.Task</*Folder*/dynamic> GetItemParentFolderAsync (string projectId, string itemId);

        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns the \&quot;parent\&quot; folder for the given item. 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <returns>Task of ApiResponse (Folder)</returns>
        System.Threading.Tasks.Task<ApiResponse</*Folder*/dynamic>> GetItemParentFolderAsyncWithHttpInfo (string projectId, string itemId);
        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns the resources (&#x60;items&#x60;, &#x60;folders&#x60;, and &#x60;versions&#x60;) which have a custom relationship with the given &#x60;item_id&#x60;. Custom relationships can be established between an item and other resources within the &#39;data&#39; domain service (folders, items, and versions). 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <returns>Task of JsonApiCollection</returns>
        System.Threading.Tasks.Task</*JsonApiCollection*/dynamic> GetItemRefsAsync (string projectId, string itemId, List<string> filterType = null, List<string> filterId = null, List<string> filterExtensionType = null);

        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns the resources (&#x60;items&#x60;, &#x60;folders&#x60;, and &#x60;versions&#x60;) which have a custom relationship with the given &#x60;item_id&#x60;. Custom relationships can be established between an item and other resources within the &#39;data&#39; domain service (folders, items, and versions). 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <returns>Task of ApiResponse (JsonApiCollection)</returns>
        System.Threading.Tasks.Task<ApiResponse</*JsonApiCollection*/dynamic>> GetItemRefsAsyncWithHttpInfo (string projectId, string itemId, List<string> filterType = null, List<string> filterId = null, List<string> filterExtensionType = null);

        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns a collection of links for the given item_id. Custom relationships can be established between an item and other external resources residing outside the data domain service. A link’s href defines the target URI to access a resource.
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterMimeType">Filter by mime type. (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <returns>Links</returns>
        System.Threading.Tasks.Task<dynamic> GetItemRelationshipsLinksAsync (string projectId, string itemId, List<string> filterType = null, List<string> filterId = null, List<string> filterMimeType = null, List<string> filterExtensionType = null);

        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns a collection of links for the given item_id. Custom relationships can be established between an item and other external resources residing outside the data domain service. A link’s href defines the target URI to access a resource.
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterMimeType">Filter by mime type. (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <returns>ApiResponse of Links</returns>
        System.Threading.Tasks.Task<ApiResponse<dynamic>> GetItemRelationshipsLinksAsyncWithHttpInfo (string projectId, string itemId, List<string> filterType = null, List<string> filterId = null, List<string> filterMimeType = null, List<string> filterExtensionType = null);

        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns the custom relationships that are associated to the given &#x60;item_id&#x60;. Custom relationships can be established between an item and other resources within the &#39;data&#39; domain service (folders, items, and versions). 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterRefType">filter by &#x60;refType&#x60; (optional)</param>
        /// <param name="filterDirection">filter by the direction of the reference (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <returns>Task of Refs</returns>
        System.Threading.Tasks.Task</*Refs*/dynamic> GetItemRelationshipsRefsAsync (string projectId, string itemId, List<string> filterType = null, List<string> filterId = null, List<string> filterRefType = null, string filterDirection = null, List<string> filterExtensionType = null);

        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns the custom relationships that are associated to the given &#x60;item_id&#x60;. Custom relationships can be established between an item and other resources within the &#39;data&#39; domain service (folders, items, and versions). 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterRefType">filter by &#x60;refType&#x60; (optional)</param>
        /// <param name="filterDirection">filter by the direction of the reference (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <returns>Task of ApiResponse (Refs)</returns>
        System.Threading.Tasks.Task<ApiResponse</*Refs*/dynamic>> GetItemRelationshipsRefsAsyncWithHttpInfo (string projectId, string itemId, List<string> filterType = null, List<string> filterId = null, List<string> filterRefType = null, string filterDirection = null, List<string> filterExtensionType = null);
        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns the \&quot;tip\&quot; version for the given item. Multiple versions of a resource item can be uploaded in a project. The tip version is the most recent one. 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <returns>Task of Version</returns>
        System.Threading.Tasks.Task</*Version*/dynamic> GetItemTipAsync (string projectId, string itemId);

        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns the \&quot;tip\&quot; version for the given item. Multiple versions of a resource item can be uploaded in a project. The tip version is the most recent one. 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <returns>Task of ApiResponse (Version)</returns>
        System.Threading.Tasks.Task<ApiResponse</*Version*/dynamic>> GetItemTipAsyncWithHttpInfo (string projectId, string itemId);
        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns versions for the given item. Multiple versions of a resource item can be uploaded in a project. 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <param name="filterVersionNumber">filter by &#x60;versionNumber&#x60; (optional)</param>
        /// <param name="pageNumber">specify the page number (optional)</param>
        /// <param name="pageLimit">specify the maximal number of elements per page (optional)</param>
        /// <returns>Task of Versions</returns>
        System.Threading.Tasks.Task</*Versions*/dynamic> GetItemVersionsAsync (string projectId, string itemId, List<string> filterType = null, List<string> filterId = null, List<string> filterExtensionType = null, List<int?> filterVersionNumber = null, int? pageNumber = null, int? pageLimit = null);

        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns versions for the given item. Multiple versions of a resource item can be uploaded in a project. 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <param name="filterVersionNumber">filter by &#x60;versionNumber&#x60; (optional)</param>
        /// <param name="pageNumber">specify the page number (optional)</param>
        /// <param name="pageLimit">specify the maximal number of elements per page (optional)</param>
        /// <returns>Task of ApiResponse (Versions)</returns>
        System.Threading.Tasks.Task<ApiResponse</*Versions*/dynamic>> GetItemVersionsAsyncWithHttpInfo (string projectId, string itemId, List<string> filterType = null, List<string> filterId = null, List<string> filterExtensionType = null, List<int?> filterVersionNumber = null, int? pageNumber = null, int? pageLimit = null);
        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Creates a new item in the &#39;data&#39; domain service. 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="body">describe the item to be created</param>
        /// <returns>Task of ItemCreated</returns>
        System.Threading.Tasks.Task</*ItemCreated*/dynamic> PostItemAsync (string projectId, CreateItem body);

        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Creates a new item in the &#39;data&#39; domain service. 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="body">describe the item to be created</param>
        /// <returns>Task of ApiResponse (ItemCreated)</returns>
        System.Threading.Tasks.Task<ApiResponse</*ItemCreated*/dynamic>> PostItemAsyncWithHttpInfo (string projectId, CreateItem body);
        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Creates a custom relationship between an item and another resource within the &#39;data&#39; domain service (folder, item, or version). 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <param name="body">describe the ref to be created</param>
        /// <returns>Task of void</returns>
        System.Threading.Tasks.Task<dynamic> PostItemRelationshipsRefAsync (string projectId, string itemId, CreateRef body);

        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Creates a custom relationship between an item and another resource within the &#39;data&#39; domain service (folder, item, or version). 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <param name="body">describe the ref to be created</param>
        /// <returns>Task of ApiResponse</returns>
        System.Threading.Tasks.Task<ApiResponse<dynamic>> PostItemRelationshipsRefAsyncWithHttpInfo (string projectId, string itemId, CreateRef body);

        /// <summary>
        ///  Updates the properties of the given item_id object.
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <param name="body">describe the ref to be created</param>
        /// <returns>Task of void</returns>
        System.Threading.Tasks.Task<dynamic> PatchItemAsync (string projectId, string itemId, PatchItem body);

        /// <summary>
        /// Updates the properties of the given item_id object.
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <param name="body">describe the ref to be created</param>
        /// <returns>Task of ApiResponse</returns>
        System.Threading.Tasks.Task<ApiResponse<dynamic>> PatchItemAsyncWithHttpInfo (string projectId, string itemId, PatchItem body);

        #endregion Asynchronous Operations
    }

    /// <summary>
    /// Represents a collection of functions to interact with the API endpoints
    /// </summary>
    public partial class ItemsApi : IItemsApi
    {
        private Autodesk.Forge.Client.ExceptionFactory _exceptionFactory = (name, response) => null;

        /// <summary>
        /// Initializes a new instance of the <see cref="ItemsApi"/> class.
        /// </summary>
        /// <returns></returns>
        public ItemsApi(String basePath)
        {
            this.Configuration = new Configuration(new ApiClient(basePath));

            ExceptionFactory = Autodesk.Forge.Client.Configuration.DefaultExceptionFactory;

            // ensure API client has configuration ready
            if (Configuration.ApiClient.Configuration == null)
            {
                this.Configuration.ApiClient.Configuration = this.Configuration;
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ItemsApi"/> class
        /// using Configuration object
        /// </summary>
        /// <param name="configuration">An instance of Configuration</param>
        /// <returns></returns>
        public ItemsApi(Configuration configuration = null)
        {
            if (configuration == null) // use the default one in Configuration
                this.Configuration = Configuration.Default;
            else
                this.Configuration = configuration;

            ExceptionFactory = Autodesk.Forge.Client.Configuration.DefaultExceptionFactory;

            // ensure API client has configuration ready
            if (Configuration.ApiClient.Configuration == null)
            {
                this.Configuration.ApiClient.Configuration = this.Configuration;
            }
        }

        /// <summary>
        /// Gets the base path of the API client.
        /// </summary>
        /// <value>The base path</value>
        public String GetBasePath()
        {
            return this.Configuration.ApiClient.RestClient.BaseUrl.ToString();
        }

        /// <summary>
        /// Sets the base path of the API client.
        /// </summary>
        /// <value>The base path</value>
        [Obsolete("SetBasePath is deprecated, please do 'Configuration.ApiClient = new ApiClient(\"http://new-path\")' instead.")]
        public void SetBasePath(String basePath)
        {
            // do nothing
        }

        /// <summary>
        /// Gets or sets the configuration object
        /// </summary>
        /// <value>An instance of the Configuration</value>
        public Configuration Configuration {get; set;}

        /// <summary>
        /// Provides a factory method hook for the creation of exceptions.
        /// </summary>
        public Autodesk.Forge.Client.ExceptionFactory ExceptionFactory
        {
            get
            {
                if (_exceptionFactory != null && _exceptionFactory.GetInvocationList().Length > 1)
                {
                    throw new InvalidOperationException("Multicast delegate for ExceptionFactory is unsupported.");
                }
                return _exceptionFactory;
            }
            set { _exceptionFactory = value; }
        }

        /// <summary>
        /// Gets the default header.
        /// </summary>
        /// <returns>Dictionary of HTTP header</returns>
        [Obsolete("DefaultHeader is deprecated, please use Configuration.DefaultHeader instead.")]
        public Dictionary<String, String> DefaultHeader()
        {
            return this.Configuration.DefaultHeader;
        }

        /// <summary>
        /// Add default header.
        /// </summary>
        /// <param name="key">Header field name.</param>
        /// <param name="value">Header field value.</param>
        /// <returns></returns>
        [Obsolete("AddDefaultHeader is deprecated, please use Configuration.AddDefaultHeader instead.")]
        public void AddDefaultHeader(string key, string value)
        {
            this.Configuration.AddDefaultHeader(key, value);
        }

        /// <summary>
        ///  Returns a resource item by ID for any item within a given project. Resource items represent word documents, fusion design files, drawings, spreadsheets, etc. 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <returns>Item</returns>
        public /*Item*/dynamic GetItem (string projectId, string itemId)
        {
             ApiResponse</*Item*/dynamic> localVarResponse = GetItemWithHttpInfo(projectId, itemId);
             return localVarResponse.Data;
        }

        /// <summary>
        ///  Returns a resource item by ID for any item within a given project. Resource items represent word documents, fusion design files, drawings, spreadsheets, etc. 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <returns>ApiResponse of Item</returns>
        public ApiResponse< /*Item*/dynamic > GetItemWithHttpInfo (string projectId, string itemId)
        {
            // verify the required parameter 'projectId' is set
            if (projectId == null)
                throw new ApiException(400, "Missing required parameter 'projectId' when calling ItemsApi->GetItem");
            // verify the required parameter 'itemId' is set
            if (itemId == null)
                throw new ApiException(400, "Missing required parameter 'itemId' when calling ItemsApi->GetItem");

            var localVarPath = "/data/v1/projects/{project_id}/items/{item_id}";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new Dictionary<String, String>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/vnd.api+json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/vnd.api+json", 
                "application/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            // set "format" to json by default
            // e.g. /pet/{petId}.{format} becomes /pet/{petId}.json
            localVarPathParams.Add("format", "json");
            if (projectId != null) localVarPathParams.Add("project_id", Configuration.ApiClient.ParameterToString(projectId)); // path parameter
            if (itemId != null) localVarPathParams.Add("item_id", Configuration.ApiClient.ParameterToString(itemId)); // path parameter

            // authentication (oauth2_access_code) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) Configuration.ApiClient.CallApi(localVarPath,
                Method.GET, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("GetItem", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse</*Item*/dynamic>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                /*(Item)*/ Configuration.ApiClient.Deserialize(localVarResponse, typeof(Item)));
            
        }

        /// <summary>
        ///  Returns a resource item by ID for any item within a given project. Resource items represent word documents, fusion design files, drawings, spreadsheets, etc. 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <returns>Task of Item</returns>
        public async System.Threading.Tasks.Task</*Item*/dynamic> GetItemAsync (string projectId, string itemId)
        {
             ApiResponse</*Item*/dynamic> localVarResponse = await GetItemAsyncWithHttpInfo(projectId, itemId);
             return localVarResponse.Data;
        }

        /// <summary>
        ///  Returns a resource item by ID for any item within a given project. Resource items represent word documents, fusion design files, drawings, spreadsheets, etc. 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <returns>Task of ApiResponse (Item)</returns>
        public async System.Threading.Tasks.Task<ApiResponse</*Item*/dynamic>> GetItemAsyncWithHttpInfo (string projectId, string itemId)
        {
            // verify the required parameter 'projectId' is set
            if (projectId == null)
                throw new ApiException(400, "Missing required parameter 'projectId' when calling ItemsApi->GetItem");
            // verify the required parameter 'itemId' is set
            if (itemId == null)
                throw new ApiException(400, "Missing required parameter 'itemId' when calling ItemsApi->GetItem");

            var localVarPath = "/data/v1/projects/{project_id}/items/{item_id}";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new Dictionary<String, String>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/vnd.api+json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/vnd.api+json", 
                "application/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            // set "format" to json by default
            // e.g. /pet/{petId}.{format} becomes /pet/{petId}.json
            localVarPathParams.Add("format", "json");
            if (projectId != null) localVarPathParams.Add("project_id", Configuration.ApiClient.ParameterToString(projectId)); // path parameter
            if (itemId != null) localVarPathParams.Add("item_id", Configuration.ApiClient.ParameterToString(itemId)); // path parameter

            // authentication (oauth2_access_code) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) await Configuration.ApiClient.CallApiAsync(localVarPath,
                Method.GET, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("GetItem", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse</*Item*/dynamic>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                /*(Item)*/ Configuration.ApiClient.Deserialize(localVarResponse, typeof(Item)));
            
        }

        /// <summary>
        ///  Returns the \&quot;parent\&quot; folder for the given item. 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <returns>Folder</returns>
        public /*Folder*/dynamic GetItemParentFolder (string projectId, string itemId)
        {
             ApiResponse</*Folder*/dynamic> localVarResponse = GetItemParentFolderWithHttpInfo(projectId, itemId);
             return localVarResponse.Data;
        }

        /// <summary>
        ///  Returns the \&quot;parent\&quot; folder for the given item. 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <returns>ApiResponse of Folder</returns>
        public ApiResponse< /*Folder*/dynamic > GetItemParentFolderWithHttpInfo (string projectId, string itemId)
        {
            // verify the required parameter 'projectId' is set
            if (projectId == null)
                throw new ApiException(400, "Missing required parameter 'projectId' when calling ItemsApi->GetItemParentFolder");
            // verify the required parameter 'itemId' is set
            if (itemId == null)
                throw new ApiException(400, "Missing required parameter 'itemId' when calling ItemsApi->GetItemParentFolder");

            var localVarPath = "/data/v1/projects/{project_id}/items/{item_id}/parent";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new Dictionary<String, String>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/vnd.api+json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/vnd.api+json", 
                "application/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            // set "format" to json by default
            // e.g. /pet/{petId}.{format} becomes /pet/{petId}.json
            localVarPathParams.Add("format", "json");
            if (projectId != null) localVarPathParams.Add("project_id", Configuration.ApiClient.ParameterToString(projectId)); // path parameter
            if (itemId != null) localVarPathParams.Add("item_id", Configuration.ApiClient.ParameterToString(itemId)); // path parameter

            // authentication (oauth2_access_code) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) Configuration.ApiClient.CallApi(localVarPath,
                Method.GET, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("GetItemParentFolder", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse</*Folder*/dynamic>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                /*(Folder)*/ Configuration.ApiClient.Deserialize(localVarResponse, typeof(Folder)));
            
        }

        /// <summary>
        ///  Returns the \&quot;parent\&quot; folder for the given item. 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <returns>Task of Folder</returns>
        public async System.Threading.Tasks.Task</*Folder*/dynamic> GetItemParentFolderAsync (string projectId, string itemId)
        {
             ApiResponse</*Folder*/dynamic> localVarResponse = await GetItemParentFolderAsyncWithHttpInfo(projectId, itemId);
             return localVarResponse.Data;
        }

        /// <summary>
        ///  Returns the \&quot;parent\&quot; folder for the given item. 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <returns>Task of ApiResponse (Folder)</returns>
        public async System.Threading.Tasks.Task<ApiResponse</*Folder*/dynamic>> GetItemParentFolderAsyncWithHttpInfo (string projectId, string itemId)
        {
            // verify the required parameter 'projectId' is set
            if (projectId == null)
                throw new ApiException(400, "Missing required parameter 'projectId' when calling ItemsApi->GetItemParentFolder");
            // verify the required parameter 'itemId' is set
            if (itemId == null)
                throw new ApiException(400, "Missing required parameter 'itemId' when calling ItemsApi->GetItemParentFolder");

            var localVarPath = "/data/v1/projects/{project_id}/items/{item_id}/parent";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new Dictionary<String, String>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/vnd.api+json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/vnd.api+json", 
                "application/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            // set "format" to json by default
            // e.g. /pet/{petId}.{format} becomes /pet/{petId}.json
            localVarPathParams.Add("format", "json");
            if (projectId != null) localVarPathParams.Add("project_id", Configuration.ApiClient.ParameterToString(projectId)); // path parameter
            if (itemId != null) localVarPathParams.Add("item_id", Configuration.ApiClient.ParameterToString(itemId)); // path parameter

            // authentication (oauth2_access_code) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) await Configuration.ApiClient.CallApiAsync(localVarPath,
                Method.GET, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("GetItemParentFolder", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse</*Folder*/dynamic>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                /*(Folder)*/ Configuration.ApiClient.Deserialize(localVarResponse, typeof(Folder)));
            
        }

        /// <summary>
        ///  Returns the resources (&#x60;items&#x60;, &#x60;folders&#x60;, and &#x60;versions&#x60;) which have a custom relationship with the given &#x60;item_id&#x60;. Custom relationships can be established between an item and other resources within the &#39;data&#39; domain service (folders, items, and versions). 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <returns>JsonApiCollection</returns>
        public /*JsonApiCollection*/dynamic GetItemRefs (string projectId, string itemId, List<string> filterType = null, List<string> filterId = null, List<string> filterExtensionType = null)
        {
             ApiResponse</*JsonApiCollection*/dynamic> localVarResponse = GetItemRefsWithHttpInfo(projectId, itemId, filterType, filterId, filterExtensionType);
             return localVarResponse.Data;
        }

        /// <summary>
        ///  Returns the resources (&#x60;items&#x60;, &#x60;folders&#x60;, and &#x60;versions&#x60;) which have a custom relationship with the given &#x60;item_id&#x60;. Custom relationships can be established between an item and other resources within the &#39;data&#39; domain service (folders, items, and versions). 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <returns>ApiResponse of JsonApiCollection</returns>
        public ApiResponse< /*JsonApiCollection*/dynamic > GetItemRefsWithHttpInfo (string projectId, string itemId, List<string> filterType = null, List<string> filterId = null, List<string> filterExtensionType = null)
        {
            // verify the required parameter 'projectId' is set
            if (projectId == null)
                throw new ApiException(400, "Missing required parameter 'projectId' when calling ItemsApi->GetItemRefs");
            // verify the required parameter 'itemId' is set
            if (itemId == null)
                throw new ApiException(400, "Missing required parameter 'itemId' when calling ItemsApi->GetItemRefs");

            var localVarPath = "/data/v1/projects/{project_id}/items/{item_id}/refs";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new Dictionary<String, String>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/vnd.api+json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/vnd.api+json", 
                "application/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            // set "format" to json by default
            // e.g. /pet/{petId}.{format} becomes /pet/{petId}.json
            localVarPathParams.Add("format", "json");
            if (projectId != null) localVarPathParams.Add("project_id", Configuration.ApiClient.ParameterToString(projectId)); // path parameter
            if (itemId != null) localVarPathParams.Add("item_id", Configuration.ApiClient.ParameterToString(itemId)); // path parameter
            if (filterType != null) localVarQueryParams.Add("filter[type]", Configuration.ApiClient.ParameterToString(filterType)); // query parameter
            if (filterId != null) localVarQueryParams.Add("filter[id]", Configuration.ApiClient.ParameterToString(filterId)); // query parameter
            if (filterExtensionType != null) localVarQueryParams.Add("filter[extension.type]", Configuration.ApiClient.ParameterToString(filterExtensionType)); // query parameter

            // authentication (oauth2_access_code) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) Configuration.ApiClient.CallApi(localVarPath,
                Method.GET, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("GetItemRefs", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse</*JsonApiCollection*/dynamic>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                /*(JsonApiCollection)*/ Configuration.ApiClient.Deserialize(localVarResponse, typeof(JsonApiCollection)));
            
        }

        /// <summary>
        ///  Returns the resources (&#x60;items&#x60;, &#x60;folders&#x60;, and &#x60;versions&#x60;) which have a custom relationship with the given &#x60;item_id&#x60;. Custom relationships can be established between an item and other resources within the &#39;data&#39; domain service (folders, items, and versions). 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <returns>Task of JsonApiCollection</returns>
        public async System.Threading.Tasks.Task</*JsonApiCollection*/dynamic> GetItemRefsAsync (string projectId, string itemId, List<string> filterType = null, List<string> filterId = null, List<string> filterExtensionType = null)
        {
             ApiResponse</*JsonApiCollection*/dynamic> localVarResponse = await GetItemRefsAsyncWithHttpInfo(projectId, itemId, filterType, filterId, filterExtensionType);
             return localVarResponse.Data;
        }

        /// <summary>
        ///  Returns the resources (&#x60;items&#x60;, &#x60;folders&#x60;, and &#x60;versions&#x60;) which have a custom relationship with the given &#x60;item_id&#x60;. Custom relationships can be established between an item and other resources within the &#39;data&#39; domain service (folders, items, and versions). 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <returns>Task of ApiResponse (JsonApiCollection)</returns>
        public async System.Threading.Tasks.Task<ApiResponse</*JsonApiCollection*/dynamic>> GetItemRefsAsyncWithHttpInfo (string projectId, string itemId, List<string> filterType = null, List<string> filterId = null, List<string> filterExtensionType = null)
        {
            // verify the required parameter 'projectId' is set
            if (projectId == null)
                throw new ApiException(400, "Missing required parameter 'projectId' when calling ItemsApi->GetItemRefs");
            // verify the required parameter 'itemId' is set
            if (itemId == null)
                throw new ApiException(400, "Missing required parameter 'itemId' when calling ItemsApi->GetItemRefs");

            var localVarPath = "/data/v1/projects/{project_id}/items/{item_id}/refs";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new Dictionary<String, String>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/vnd.api+json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/vnd.api+json", 
                "application/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            // set "format" to json by default
            // e.g. /pet/{petId}.{format} becomes /pet/{petId}.json
            localVarPathParams.Add("format", "json");
            if (projectId != null) localVarPathParams.Add("project_id", Configuration.ApiClient.ParameterToString(projectId)); // path parameter
            if (itemId != null) localVarPathParams.Add("item_id", Configuration.ApiClient.ParameterToString(itemId)); // path parameter
            if (filterType != null) localVarQueryParams.Add("filter[type]", Configuration.ApiClient.ParameterToString(filterType)); // query parameter
            if (filterId != null) localVarQueryParams.Add("filter[id]", Configuration.ApiClient.ParameterToString(filterId)); // query parameter
            if (filterExtensionType != null) localVarQueryParams.Add("filter[extension.type]", Configuration.ApiClient.ParameterToString(filterExtensionType)); // query parameter

            // authentication (oauth2_access_code) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) await Configuration.ApiClient.CallApiAsync(localVarPath,
                Method.GET, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("GetItemRefs", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse</*JsonApiCollection*/dynamic>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                /*(JsonApiCollection)*/ Configuration.ApiClient.Deserialize(localVarResponse, typeof(JsonApiCollection)));
            
        }

        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns a collection of links for the given item_id. Custom relationships can be established between an item and other external resources residing outside the data domain service. A link’s href defines the target URI to access a resource.
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterMimeType">Filter by mime type. (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <returns>Links</returns>
        /*Links*/
        public dynamic GetItemRelationshipsLinks (string projectId, string itemId, List<string> filterType = null, List<string> filterId = null, List<string> filterMimeType = null, List<string> filterExtensionType = null) {
            ApiResponse</*Refs*/dynamic> localVarResponse = GetItemRelationshipsLinksWithHttpInfo (projectId, itemId, filterType, filterId, filterMimeType, filterExtensionType);
            return localVarResponse.Data;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns a collection of links for the given item_id. Custom relationships can be established between an item and other external resources residing outside the data domain service. A link’s href defines the target URI to access a resource.
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterMimeType">Filter by mime type. (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <returns>ApiResponse of Links</returns>
        public ApiResponse</*Refs*/dynamic> GetItemRelationshipsLinksWithHttpInfo (string projectId, string itemId, List<string> filterType = null, List<string> filterId = null, List<string> filterMimeType = null, List<string> filterExtensionType = null) {
            // verify the required parameter 'projectId' is set
            if ( projectId == null )
                throw new ApiException (400, "Missing required parameter 'projectId' when calling ItemsApi->GetItemRelationshipsLinks");
            // verify the required parameter 'itemId' is set
            if ( itemId == null )
                throw new ApiException (400, "Missing required parameter 'itemId' when calling ItemsApi->GetItemRelationshipsLinks");

            var localVarPath = "/data/v1/projects/{project_id}/items/{item_id}/relationships/links";
            var localVarPathParams = new Dictionary<String, String> ();
            var localVarQueryParams = new Dictionary<String, String> ();
            var localVarHeaderParams = new Dictionary<String, String> (Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String> ();
            var localVarFileParams = new Dictionary<String, FileParameter> ();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String [] localVarHttpContentTypes = new String [] {
                "application/vnd.api+json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType (localVarHttpContentTypes);

            // to determine the Accept header
            String [] localVarHttpHeaderAccepts = new String [] {
                "application/vnd.api+json",
                "application/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept (localVarHttpHeaderAccepts);
            if ( localVarHttpHeaderAccept != null )
                localVarHeaderParams.Add ("Accept", localVarHttpHeaderAccept);

            // set "format" to json by default
            // e.g. /pet/{petId}.{format} becomes /pet/{petId}.json
            localVarPathParams.Add ("format", "json");
            if ( projectId != null )
                localVarPathParams.Add ("project_id", Configuration.ApiClient.ParameterToString (projectId)); // path parameter
            if ( itemId != null )
                localVarPathParams.Add ("item_id", Configuration.ApiClient.ParameterToString (itemId)); // path parameter
            if ( filterType != null )
                localVarQueryParams.Add ("filter[type]", Configuration.ApiClient.ParameterToString (filterType)); // query parameter
            if ( filterId != null )
                localVarQueryParams.Add ("filter[id]", Configuration.ApiClient.ParameterToString (filterId)); // query parameter
            if ( filterMimeType != null )
                localVarQueryParams.Add ("filter[mimeType]", Configuration.ApiClient.ParameterToString (filterMimeType)); // query parameter
            if ( filterExtensionType != null )
                localVarQueryParams.Add ("filter[extension.type]", Configuration.ApiClient.ParameterToString (filterExtensionType)); // query parameter

            // authentication (oauth2_access_code) required
            // oauth required
            if ( !String.IsNullOrEmpty (Configuration.AccessToken) ) {
                localVarHeaderParams ["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse)Configuration.ApiClient.CallApi (localVarPath,
                Method.GET, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int)localVarResponse.StatusCode;

            if ( ExceptionFactory != null ) {
                Exception exception = ExceptionFactory ("GetItemRelationshipsLinks", localVarResponse);
                if ( exception != null )
                    throw exception;
            }

            return new ApiResponse</*Links*/dynamic> (localVarStatusCode,
                localVarResponse.Headers.ToDictionary (x => x.Name, x => x.Value.ToString ()),
                /*(Links)*/ Configuration.ApiClient.Deserialize (localVarResponse, typeof (Refs)));
        }

        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns a collection of links for the given item_id. Custom relationships can be established between an item and other external resources residing outside the data domain service. A link’s href defines the target URI to access a resource.
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterMimeType">Filter by mime type. (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <returns>Links</returns>
        public async System.Threading.Tasks.Task<dynamic> GetItemRelationshipsLinksAsync (string projectId, string itemId, List<string> filterType = null, List<string> filterId = null, List<string> filterMimeType = null, List<string> filterExtensionType = null)
        {
             ApiResponse</*Refs*/dynamic> localVarResponse = await GetItemRelationshipsLinksAsyncWithHttpInfo (projectId, itemId, filterType, filterId, filterMimeType, filterExtensionType);
             return localVarResponse.Data;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns a collection of links for the given item_id. Custom relationships can be established between an item and other external resources residing outside the data domain service. A link’s href defines the target URI to access a resource.
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterMimeType">Filter by mime type. (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <returns>ApiResponse of Links</returns>
        public async System.Threading.Tasks.Task<ApiResponse<dynamic>> GetItemRelationshipsLinksAsyncWithHttpInfo (string projectId, string itemId, List<string> filterType = null, List<string> filterId = null, List<string> filterMimeType = null, List<string> filterExtensionType = null) {
            // verify the required parameter 'projectId' is set
            if ( projectId == null )
                throw new ApiException (400, "Missing required parameter 'projectId' when calling ItemsApi->GetItemRelationshipsLinks");
            // verify the required parameter 'itemId' is set
            if ( itemId == null )
                throw new ApiException (400, "Missing required parameter 'itemId' when calling ItemsApi->GetItemRelationshipsLinks");

            var localVarPath = "/data/v1/projects/{project_id}/items/{item_id}/relationships/links";
            var localVarPathParams = new Dictionary<String, String> ();
            var localVarQueryParams = new Dictionary<String, String> ();
            var localVarHeaderParams = new Dictionary<String, String> (Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String> ();
            var localVarFileParams = new Dictionary<String, FileParameter> ();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String [] localVarHttpContentTypes = new String [] {
                "application/vnd.api+json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType (localVarHttpContentTypes);

            // to determine the Accept header
            String [] localVarHttpHeaderAccepts = new String [] {
                "application/vnd.api+json",
                "application/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept (localVarHttpHeaderAccepts);
            if ( localVarHttpHeaderAccept != null )
                localVarHeaderParams.Add ("Accept", localVarHttpHeaderAccept);

            // set "format" to json by default
            // e.g. /pet/{petId}.{format} becomes /pet/{petId}.json
            localVarPathParams.Add ("format", "json");
            if ( projectId != null )
                localVarPathParams.Add ("project_id", Configuration.ApiClient.ParameterToString (projectId)); // path parameter
            if ( itemId != null )
                localVarPathParams.Add ("item_id", Configuration.ApiClient.ParameterToString (itemId)); // path parameter
            if ( filterType != null )
                localVarQueryParams.Add ("filter[type]", Configuration.ApiClient.ParameterToString (filterType)); // query parameter
            if ( filterId != null )
                localVarQueryParams.Add ("filter[id]", Configuration.ApiClient.ParameterToString (filterId)); // query parameter
            if ( filterMimeType != null )
                localVarQueryParams.Add ("filter[mimeType]", Configuration.ApiClient.ParameterToString (filterMimeType)); // query parameter
           if ( filterExtensionType != null )
                localVarQueryParams.Add ("filter[extension.type]", Configuration.ApiClient.ParameterToString (filterExtensionType)); // query parameter

            // authentication (oauth2_access_code) required
            // oauth required
            if ( !String.IsNullOrEmpty (Configuration.AccessToken) ) {
                localVarHeaderParams ["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse)await Configuration.ApiClient.CallApiAsync (localVarPath,
                Method.GET, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int)localVarResponse.StatusCode;

            if ( ExceptionFactory != null ) {
                Exception exception = ExceptionFactory ("GetItemRelationshipsRefs", localVarResponse);
                if ( exception != null )
                    throw exception;
            }

            return new ApiResponse</*Links*/dynamic> (localVarStatusCode,
                localVarResponse.Headers.ToDictionary (x => x.Name, x => x.Value.ToString ()),
                /*(Links)*/ Configuration.ApiClient.Deserialize (localVarResponse, typeof (Refs)));
        }

        /// <summary>
        ///  Returns the custom relationships that are associated to the given &#x60;item_id&#x60;. Custom relationships can be established between an item and other resources within the &#39;data&#39; domain service (folders, items, and versions). 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterRefType">filter by &#x60;refType&#x60; (optional)</param>
        /// <param name="filterDirection">filter by the direction of the reference (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <returns>Refs</returns>
        public /*Refs*/dynamic GetItemRelationshipsRefs (string projectId, string itemId, List<string> filterType = null, List<string> filterId = null, List<string> filterRefType = null, string filterDirection = null, List<string> filterExtensionType = null)
        {
             ApiResponse</*Refs*/dynamic> localVarResponse = GetItemRelationshipsRefsWithHttpInfo(projectId, itemId, filterType, filterId, filterRefType, filterDirection, filterExtensionType);
             return localVarResponse.Data;
        }

        /// <summary>
        ///  Returns the custom relationships that are associated to the given &#x60;item_id&#x60;. Custom relationships can be established between an item and other resources within the &#39;data&#39; domain service (folders, items, and versions). 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterRefType">filter by &#x60;refType&#x60; (optional)</param>
        /// <param name="filterDirection">filter by the direction of the reference (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <returns>ApiResponse of Refs</returns>
        public ApiResponse< /*Refs*/dynamic > GetItemRelationshipsRefsWithHttpInfo (string projectId, string itemId, List<string> filterType = null, List<string> filterId = null, List<string> filterRefType = null, string filterDirection = null, List<string> filterExtensionType = null)
        {
            // verify the required parameter 'projectId' is set
            if (projectId == null)
                throw new ApiException(400, "Missing required parameter 'projectId' when calling ItemsApi->GetItemRelationshipsRefs");
            // verify the required parameter 'itemId' is set
            if (itemId == null)
                throw new ApiException(400, "Missing required parameter 'itemId' when calling ItemsApi->GetItemRelationshipsRefs");

            var localVarPath = "/data/v1/projects/{project_id}/items/{item_id}/relationships/refs";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new Dictionary<String, String>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/vnd.api+json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/vnd.api+json", 
                "application/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            // set "format" to json by default
            // e.g. /pet/{petId}.{format} becomes /pet/{petId}.json
            localVarPathParams.Add("format", "json");
            if (projectId != null) localVarPathParams.Add("project_id", Configuration.ApiClient.ParameterToString(projectId)); // path parameter
            if (itemId != null) localVarPathParams.Add("item_id", Configuration.ApiClient.ParameterToString(itemId)); // path parameter
            if (filterType != null) localVarQueryParams.Add("filter[type]", Configuration.ApiClient.ParameterToString(filterType)); // query parameter
            if (filterId != null) localVarQueryParams.Add("filter[id]", Configuration.ApiClient.ParameterToString(filterId)); // query parameter
            if (filterRefType != null) localVarQueryParams.Add("filter[refType]", Configuration.ApiClient.ParameterToString(filterRefType)); // query parameter
            if (filterDirection != null) localVarQueryParams.Add("filter[direction]", Configuration.ApiClient.ParameterToString(filterDirection)); // query parameter
            if (filterExtensionType != null) localVarQueryParams.Add("filter[extension.type]", Configuration.ApiClient.ParameterToString(filterExtensionType)); // query parameter

            // authentication (oauth2_access_code) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) Configuration.ApiClient.CallApi(localVarPath,
                Method.GET, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("GetItemRelationshipsRefs", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse</*Refs*/dynamic>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                /*(Refs)*/ Configuration.ApiClient.Deserialize(localVarResponse, typeof(Refs)));
            
        }

        /// <summary>
        ///  Returns the custom relationships that are associated to the given &#x60;item_id&#x60;. Custom relationships can be established between an item and other resources within the &#39;data&#39; domain service (folders, items, and versions). 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterRefType">filter by &#x60;refType&#x60; (optional)</param>
        /// <param name="filterDirection">filter by the direction of the reference (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <returns>Task of Refs</returns>
        public async System.Threading.Tasks.Task</*Refs*/dynamic> GetItemRelationshipsRefsAsync (string projectId, string itemId, List<string> filterType = null, List<string> filterId = null, List<string> filterRefType = null, string filterDirection = null, List<string> filterExtensionType = null)
        {
             ApiResponse</*Refs*/dynamic> localVarResponse = await GetItemRelationshipsRefsAsyncWithHttpInfo(projectId, itemId, filterType, filterId, filterRefType, filterDirection, filterExtensionType);
             return localVarResponse.Data;
        }

        /// <summary>
        ///  Returns the custom relationships that are associated to the given &#x60;item_id&#x60;. Custom relationships can be established between an item and other resources within the &#39;data&#39; domain service (folders, items, and versions). 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterRefType">filter by &#x60;refType&#x60; (optional)</param>
        /// <param name="filterDirection">filter by the direction of the reference (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <returns>Task of ApiResponse (Refs)</returns>
        public async System.Threading.Tasks.Task<ApiResponse</*Refs*/dynamic>> GetItemRelationshipsRefsAsyncWithHttpInfo (string projectId, string itemId, List<string> filterType = null, List<string> filterId = null, List<string> filterRefType = null, string filterDirection = null, List<string> filterExtensionType = null)
        {
            // verify the required parameter 'projectId' is set
            if (projectId == null)
                throw new ApiException(400, "Missing required parameter 'projectId' when calling ItemsApi->GetItemRelationshipsRefs");
            // verify the required parameter 'itemId' is set
            if (itemId == null)
                throw new ApiException(400, "Missing required parameter 'itemId' when calling ItemsApi->GetItemRelationshipsRefs");

            var localVarPath = "/data/v1/projects/{project_id}/items/{item_id}/relationships/refs";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new Dictionary<String, String>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/vnd.api+json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/vnd.api+json", 
                "application/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            // set "format" to json by default
            // e.g. /pet/{petId}.{format} becomes /pet/{petId}.json
            localVarPathParams.Add("format", "json");
            if (projectId != null) localVarPathParams.Add("project_id", Configuration.ApiClient.ParameterToString(projectId)); // path parameter
            if (itemId != null) localVarPathParams.Add("item_id", Configuration.ApiClient.ParameterToString(itemId)); // path parameter
            if (filterType != null) localVarQueryParams.Add("filter[type]", Configuration.ApiClient.ParameterToString(filterType)); // query parameter
            if (filterId != null) localVarQueryParams.Add("filter[id]", Configuration.ApiClient.ParameterToString(filterId)); // query parameter
            if (filterRefType != null) localVarQueryParams.Add("filter[refType]", Configuration.ApiClient.ParameterToString(filterRefType)); // query parameter
            if (filterDirection != null) localVarQueryParams.Add("filter[direction]", Configuration.ApiClient.ParameterToString(filterDirection)); // query parameter
            if (filterExtensionType != null) localVarQueryParams.Add("filter[extension.type]", Configuration.ApiClient.ParameterToString(filterExtensionType)); // query parameter

            // authentication (oauth2_access_code) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) await Configuration.ApiClient.CallApiAsync(localVarPath,
                Method.GET, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("GetItemRelationshipsRefs", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse</*Refs*/dynamic>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                /*(Refs)*/ Configuration.ApiClient.Deserialize(localVarResponse, typeof(Refs)));
            
        }

        /// <summary>
        ///  Returns the \&quot;tip\&quot; version for the given item. Multiple versions of a resource item can be uploaded in a project. The tip version is the most recent one. 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <returns>Version</returns>
        public /*Version*/dynamic GetItemTip (string projectId, string itemId)
        {
             ApiResponse</*Version*/dynamic> localVarResponse = GetItemTipWithHttpInfo(projectId, itemId);
             return localVarResponse.Data;
        }

        /// <summary>
        ///  Returns the \&quot;tip\&quot; version for the given item. Multiple versions of a resource item can be uploaded in a project. The tip version is the most recent one. 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <returns>ApiResponse of Version</returns>
        public ApiResponse< /*Version*/dynamic > GetItemTipWithHttpInfo (string projectId, string itemId)
        {
            // verify the required parameter 'projectId' is set
            if (projectId == null)
                throw new ApiException(400, "Missing required parameter 'projectId' when calling ItemsApi->GetItemTip");
            // verify the required parameter 'itemId' is set
            if (itemId == null)
                throw new ApiException(400, "Missing required parameter 'itemId' when calling ItemsApi->GetItemTip");

            var localVarPath = "/data/v1/projects/{project_id}/items/{item_id}/tip";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new Dictionary<String, String>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/vnd.api+json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/vnd.api+json", 
                "application/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            // set "format" to json by default
            // e.g. /pet/{petId}.{format} becomes /pet/{petId}.json
            localVarPathParams.Add("format", "json");
            if (projectId != null) localVarPathParams.Add("project_id", Configuration.ApiClient.ParameterToString(projectId)); // path parameter
            if (itemId != null) localVarPathParams.Add("item_id", Configuration.ApiClient.ParameterToString(itemId)); // path parameter

            // authentication (oauth2_access_code) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) Configuration.ApiClient.CallApi(localVarPath,
                Method.GET, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("GetItemTip", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse</*Version*/dynamic>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                /*(Version)*/ Configuration.ApiClient.Deserialize(localVarResponse, typeof(Version)));
            
        }

        /// <summary>
        ///  Returns the \&quot;tip\&quot; version for the given item. Multiple versions of a resource item can be uploaded in a project. The tip version is the most recent one. 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <returns>Task of Version</returns>
        public async System.Threading.Tasks.Task</*Version*/dynamic> GetItemTipAsync (string projectId, string itemId)
        {
             ApiResponse</*Version*/dynamic> localVarResponse = await GetItemTipAsyncWithHttpInfo(projectId, itemId);
             return localVarResponse.Data;
        }

        /// <summary>
        ///  Returns the \&quot;tip\&quot; version for the given item. Multiple versions of a resource item can be uploaded in a project. The tip version is the most recent one. 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <returns>Task of ApiResponse (Version)</returns>
        public async System.Threading.Tasks.Task<ApiResponse</*Version*/dynamic>> GetItemTipAsyncWithHttpInfo (string projectId, string itemId)
        {
            // verify the required parameter 'projectId' is set
            if (projectId == null)
                throw new ApiException(400, "Missing required parameter 'projectId' when calling ItemsApi->GetItemTip");
            // verify the required parameter 'itemId' is set
            if (itemId == null)
                throw new ApiException(400, "Missing required parameter 'itemId' when calling ItemsApi->GetItemTip");

            var localVarPath = "/data/v1/projects/{project_id}/items/{item_id}/tip";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new Dictionary<String, String>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/vnd.api+json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/vnd.api+json", 
                "application/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            // set "format" to json by default
            // e.g. /pet/{petId}.{format} becomes /pet/{petId}.json
            localVarPathParams.Add("format", "json");
            if (projectId != null) localVarPathParams.Add("project_id", Configuration.ApiClient.ParameterToString(projectId)); // path parameter
            if (itemId != null) localVarPathParams.Add("item_id", Configuration.ApiClient.ParameterToString(itemId)); // path parameter

            // authentication (oauth2_access_code) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) await Configuration.ApiClient.CallApiAsync(localVarPath,
                Method.GET, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("GetItemTip", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse</*Version*/dynamic>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                /*(Version)*/ Configuration.ApiClient.Deserialize(localVarResponse, typeof(Version)));
            
        }

        /// <summary>
        ///  Returns versions for the given item. Multiple versions of a resource item can be uploaded in a project. 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <param name="filterVersionNumber">filter by &#x60;versionNumber&#x60; (optional)</param>
        /// <param name="pageNumber">specify the page number (optional)</param>
        /// <param name="pageLimit">specify the maximal number of elements per page (optional)</param>
        /// <returns>Versions</returns>
        public /*Versions*/dynamic GetItemVersions (string projectId, string itemId, List<string> filterType = null, List<string> filterId = null, List<string> filterExtensionType = null, List<int?> filterVersionNumber = null, int? pageNumber = null, int? pageLimit = null)
        {
             ApiResponse</*Versions*/dynamic> localVarResponse = GetItemVersionsWithHttpInfo(projectId, itemId, filterType, filterId, filterExtensionType, filterVersionNumber, pageNumber, pageLimit);
             return localVarResponse.Data;
        }

        /// <summary>
        ///  Returns versions for the given item. Multiple versions of a resource item can be uploaded in a project. 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <param name="filterVersionNumber">filter by &#x60;versionNumber&#x60; (optional)</param>
        /// <param name="pageNumber">specify the page number (optional)</param>
        /// <param name="pageLimit">specify the maximal number of elements per page (optional)</param>
        /// <returns>ApiResponse of Versions</returns>
        public ApiResponse< /*Versions*/dynamic > GetItemVersionsWithHttpInfo (string projectId, string itemId, List<string> filterType = null, List<string> filterId = null, List<string> filterExtensionType = null, List<int?> filterVersionNumber = null, int? pageNumber = null, int? pageLimit = null)
        {
            // verify the required parameter 'projectId' is set
            if (projectId == null)
                throw new ApiException(400, "Missing required parameter 'projectId' when calling ItemsApi->GetItemVersions");
            // verify the required parameter 'itemId' is set
            if (itemId == null)
                throw new ApiException(400, "Missing required parameter 'itemId' when calling ItemsApi->GetItemVersions");

            var localVarPath = "/data/v1/projects/{project_id}/items/{item_id}/versions";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new Dictionary<String, String>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/vnd.api+json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/vnd.api+json", 
                "application/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            // set "format" to json by default
            // e.g. /pet/{petId}.{format} becomes /pet/{petId}.json
            localVarPathParams.Add("format", "json");
            if (projectId != null) localVarPathParams.Add("project_id", Configuration.ApiClient.ParameterToString(projectId)); // path parameter
            if (itemId != null) localVarPathParams.Add("item_id", Configuration.ApiClient.ParameterToString(itemId)); // path parameter
            if (filterType != null) localVarQueryParams.Add("filter[type]", Configuration.ApiClient.ParameterToString(filterType)); // query parameter
            if (filterId != null) localVarQueryParams.Add("filter[id]", Configuration.ApiClient.ParameterToString(filterId)); // query parameter
            if (filterExtensionType != null) localVarQueryParams.Add("filter[extension.type]", Configuration.ApiClient.ParameterToString(filterExtensionType)); // query parameter
            if (filterVersionNumber != null) localVarQueryParams.Add("filter[versionNumber]", Configuration.ApiClient.ParameterToString(filterVersionNumber)); // query parameter
            if (pageNumber != null) localVarQueryParams.Add("page[number]", Configuration.ApiClient.ParameterToString(pageNumber)); // query parameter
            if (pageLimit != null) localVarQueryParams.Add("page[limit]", Configuration.ApiClient.ParameterToString(pageLimit)); // query parameter

            // authentication (oauth2_access_code) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) Configuration.ApiClient.CallApi(localVarPath,
                Method.GET, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("GetItemVersions", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse</*Versions*/dynamic>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                /*(Versions)*/ Configuration.ApiClient.Deserialize(localVarResponse, typeof(Versions)));
            
        }

        /// <summary>
        ///  Returns versions for the given item. Multiple versions of a resource item can be uploaded in a project. 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <param name="filterVersionNumber">filter by &#x60;versionNumber&#x60; (optional)</param>
        /// <param name="pageNumber">specify the page number (optional)</param>
        /// <param name="pageLimit">specify the maximal number of elements per page (optional)</param>
        /// <returns>Task of Versions</returns>
        public async System.Threading.Tasks.Task</*Versions*/dynamic> GetItemVersionsAsync (string projectId, string itemId, List<string> filterType = null, List<string> filterId = null, List<string> filterExtensionType = null, List<int?> filterVersionNumber = null, int? pageNumber = null, int? pageLimit = null)
        {
             ApiResponse</*Versions*/dynamic> localVarResponse = await GetItemVersionsAsyncWithHttpInfo(projectId, itemId, filterType, filterId, filterExtensionType, filterVersionNumber, pageNumber, pageLimit);
             return localVarResponse.Data;
        }

        /// <summary>
        ///  Returns versions for the given item. Multiple versions of a resource item can be uploaded in a project. 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <param name="filterVersionNumber">filter by &#x60;versionNumber&#x60; (optional)</param>
        /// <param name="pageNumber">specify the page number (optional)</param>
        /// <param name="pageLimit">specify the maximal number of elements per page (optional)</param>
        /// <returns>Task of ApiResponse (Versions)</returns>
        public async System.Threading.Tasks.Task<ApiResponse</*Versions*/dynamic>> GetItemVersionsAsyncWithHttpInfo (string projectId, string itemId, List<string> filterType = null, List<string> filterId = null, List<string> filterExtensionType = null, List<int?> filterVersionNumber = null, int? pageNumber = null, int? pageLimit = null)
        {
            // verify the required parameter 'projectId' is set
            if (projectId == null)
                throw new ApiException(400, "Missing required parameter 'projectId' when calling ItemsApi->GetItemVersions");
            // verify the required parameter 'itemId' is set
            if (itemId == null)
                throw new ApiException(400, "Missing required parameter 'itemId' when calling ItemsApi->GetItemVersions");

            var localVarPath = "/data/v1/projects/{project_id}/items/{item_id}/versions";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new Dictionary<String, String>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/vnd.api+json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/vnd.api+json", 
                "application/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            // set "format" to json by default
            // e.g. /pet/{petId}.{format} becomes /pet/{petId}.json
            localVarPathParams.Add("format", "json");
            if (projectId != null) localVarPathParams.Add("project_id", Configuration.ApiClient.ParameterToString(projectId)); // path parameter
            if (itemId != null) localVarPathParams.Add("item_id", Configuration.ApiClient.ParameterToString(itemId)); // path parameter
            if (filterType != null) localVarQueryParams.Add("filter[type]", Configuration.ApiClient.ParameterToString(filterType)); // query parameter
            if (filterId != null) localVarQueryParams.Add("filter[id]", Configuration.ApiClient.ParameterToString(filterId)); // query parameter
            if (filterExtensionType != null) localVarQueryParams.Add("filter[extension.type]", Configuration.ApiClient.ParameterToString(filterExtensionType)); // query parameter
            if (filterVersionNumber != null) localVarQueryParams.Add("filter[versionNumber]", Configuration.ApiClient.ParameterToString(filterVersionNumber)); // query parameter
            if (pageNumber != null) localVarQueryParams.Add("page[number]", Configuration.ApiClient.ParameterToString(pageNumber)); // query parameter
            if (pageLimit != null) localVarQueryParams.Add("page[limit]", Configuration.ApiClient.ParameterToString(pageLimit)); // query parameter

            // authentication (oauth2_access_code) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) await Configuration.ApiClient.CallApiAsync(localVarPath,
                Method.GET, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("GetItemVersions", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse</*Versions*/dynamic>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                /*(Versions)*/ Configuration.ApiClient.Deserialize(localVarResponse, typeof(Versions)));
            
        }

        /// <summary>
        ///  Creates a new item in the &#39;data&#39; domain service. 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="body">describe the item to be created</param>
        /// <returns>ItemCreated</returns>
        public /*ItemCreated*/dynamic PostItem (string projectId, CreateItem body)
        {
             ApiResponse</*ItemCreated*/dynamic> localVarResponse = PostItemWithHttpInfo(projectId, body);
             return localVarResponse.Data;
        }

        /// <summary>
        ///  Creates a new item in the &#39;data&#39; domain service. 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="body">describe the item to be created</param>
        /// <returns>ApiResponse of ItemCreated</returns>
        public ApiResponse< /*ItemCreated*/dynamic > PostItemWithHttpInfo (string projectId, CreateItem body)
        {
            // verify the required parameter 'projectId' is set
            if (projectId == null)
                throw new ApiException(400, "Missing required parameter 'projectId' when calling ItemsApi->PostItem");
            // verify the required parameter 'body' is set
            if (body == null)
                throw new ApiException(400, "Missing required parameter 'body' when calling ItemsApi->PostItem");

            var localVarPath = "/data/v1/projects/{project_id}/items";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new Dictionary<String, String>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/vnd.api+json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/vnd.api+json", 
                "application/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            // set "format" to json by default
            // e.g. /pet/{petId}.{format} becomes /pet/{petId}.json
            localVarPathParams.Add("format", "json");
            if (projectId != null) localVarPathParams.Add("project_id", Configuration.ApiClient.ParameterToString(projectId)); // path parameter
            if (body != null && body.GetType() == typeof(byte[])) // http body (model) parameter
            {
                localVarPostBody = body; // byte array
            }
            else if (   body != null
                     && (   body.GetType() == typeof(System.IO.FileStream)
                         || body.GetType() == typeof(System.IO.BinaryReader)
                         || body.GetType() == typeof(System.IO.BufferedStream)
                   	     || body.GetType() == typeof(System.IO.MemoryStream)
                   	     || body.GetType() == typeof(System.IO.StreamReader)
                   	    )
                    )
            {
                localVarPostBody =Configuration.ApiClient.toByteArray(body); // byte array
            }
            else if ( body != null )
            {
                localVarPostBody = Configuration.ApiClient.Serialize(body);
            }

            // authentication (oauth2_access_code) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) Configuration.ApiClient.CallApi(localVarPath,
                Method.POST, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("PostItem", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse</*ItemCreated*/dynamic>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                /*(ItemCreated)*/ Configuration.ApiClient.Deserialize(localVarResponse, typeof(ItemCreated)));
            
        }

        /// <summary>
        ///  Creates a new item in the &#39;data&#39; domain service. 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="body">describe the item to be created</param>
        /// <returns>Task of ItemCreated</returns>
        public async System.Threading.Tasks.Task</*ItemCreated*/dynamic> PostItemAsync (string projectId, CreateItem body)
        {
             ApiResponse</*ItemCreated*/dynamic> localVarResponse = await PostItemAsyncWithHttpInfo(projectId, body);
             return localVarResponse.Data;
        }

        /// <summary>
        ///  Creates a new item in the &#39;data&#39; domain service. 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="body">describe the item to be created</param>
        /// <returns>Task of ApiResponse (ItemCreated)</returns>
        public async System.Threading.Tasks.Task<ApiResponse</*ItemCreated*/dynamic>> PostItemAsyncWithHttpInfo (string projectId, CreateItem body)
        {
            // verify the required parameter 'projectId' is set
            if (projectId == null)
                throw new ApiException(400, "Missing required parameter 'projectId' when calling ItemsApi->PostItem");
            // verify the required parameter 'body' is set
            if (body == null)
                throw new ApiException(400, "Missing required parameter 'body' when calling ItemsApi->PostItem");

            var localVarPath = "/data/v1/projects/{project_id}/items";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new Dictionary<String, String>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/vnd.api+json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/vnd.api+json", 
                "application/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            // set "format" to json by default
            // e.g. /pet/{petId}.{format} becomes /pet/{petId}.json
            localVarPathParams.Add("format", "json");
            if (projectId != null) localVarPathParams.Add("project_id", Configuration.ApiClient.ParameterToString(projectId)); // path parameter
            if (body != null && body.GetType() == typeof(byte[])) // http body (model) parameter
            {
                localVarPostBody = body; // byte array
            }
            else if (   body != null
                     && (   body.GetType() == typeof(System.IO.FileStream)
                         || body.GetType() == typeof(System.IO.BinaryReader)
                         || body.GetType() == typeof(System.IO.BufferedStream)
                   	     || body.GetType() == typeof(System.IO.MemoryStream)
                   	     || body.GetType() == typeof(System.IO.StreamReader)
                   	    )
                    )
            {
                localVarPostBody =Configuration.ApiClient.toByteArray(body); // byte array
            }
            else if ( body != null )
            {
                localVarPostBody = Configuration.ApiClient.Serialize(body);
            }

            // authentication (oauth2_access_code) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) await Configuration.ApiClient.CallApiAsync(localVarPath,
                Method.POST, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("PostItem", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse</*ItemCreated*/dynamic>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                /*(ItemCreated)*/ Configuration.ApiClient.Deserialize(localVarResponse, typeof(ItemCreated)));
            
        }

        /// <summary>
        ///  Creates a custom relationship between an item and another resource within the &#39;data&#39; domain service (folder, item, or version). 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <param name="body">describe the ref to be created</param>
        /// <returns>ApiResponse of Object(void)</returns>
        public dynamic PostItemRelationshipsRef (string projectId, string itemId, CreateRef body)
        {
             return PostItemRelationshipsRefWithHttpInfo(projectId, itemId, body).Data;
        }

        /// <summary>
        ///  Creates a custom relationship between an item and another resource within the &#39;data&#39; domain service (folder, item, or version). 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <param name="body">describe the ref to be created</param>
        /// <returns>ApiResponse of Object(void)</returns>
        public ApiResponse<dynamic> PostItemRelationshipsRefWithHttpInfo (string projectId, string itemId, CreateRef body)
        {
            // verify the required parameter 'projectId' is set
            if (projectId == null)
                throw new ApiException(400, "Missing required parameter 'projectId' when calling ItemsApi->PostItemRelationshipsRef");
            // verify the required parameter 'itemId' is set
            if (itemId == null)
                throw new ApiException(400, "Missing required parameter 'itemId' when calling ItemsApi->PostItemRelationshipsRef");
            // verify the required parameter 'body' is set
            if (body == null)
                throw new ApiException(400, "Missing required parameter 'body' when calling ItemsApi->PostItemRelationshipsRef");

            var localVarPath = "/data/v1/projects/{project_id}/items/{item_id}/relationships/refs";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new Dictionary<String, String>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/vnd.api+json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/vnd.api+json", 
                "application/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            // set "format" to json by default
            // e.g. /pet/{petId}.{format} becomes /pet/{petId}.json
            localVarPathParams.Add("format", "json");
            if (projectId != null) localVarPathParams.Add("project_id", Configuration.ApiClient.ParameterToString(projectId)); // path parameter
            if (itemId != null) localVarPathParams.Add("item_id", Configuration.ApiClient.ParameterToString(itemId)); // path parameter
            if (body != null && body.GetType() == typeof(byte[])) // http body (model) parameter
            {
                localVarPostBody = body; // byte array
            }
            else if (   body != null
                     && (   body.GetType() == typeof(System.IO.FileStream)
                         || body.GetType() == typeof(System.IO.BinaryReader)
                         || body.GetType() == typeof(System.IO.BufferedStream)
                   	     || body.GetType() == typeof(System.IO.MemoryStream)
                   	     || body.GetType() == typeof(System.IO.StreamReader)
                   	    )
                    )
            {
                localVarPostBody =Configuration.ApiClient.toByteArray(body); // byte array
            }
            else if ( body != null )
            {
                localVarPostBody = Configuration.ApiClient.Serialize(body);
            }

            // authentication (oauth2_access_code) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) Configuration.ApiClient.CallApi(localVarPath,
                Method.POST, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("PostItemRelationshipsRef", localVarResponse);
                if (exception != null) throw exception;
            }

            
            return new ApiResponse<Object>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                null);
        }

        /// <summary>
        ///  Creates a custom relationship between an item and another resource within the &#39;data&#39; domain service (folder, item, or version). 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <param name="body">describe the ref to be created</param>
        /// <returns>Task of void</returns>
        public async System.Threading.Tasks.Task<dynamic> PostItemRelationshipsRefAsync (string projectId, string itemId, CreateRef body)
        {
            ApiResponse<dynamic> localVarResponse = await PostItemRelationshipsRefAsyncWithHttpInfo(projectId, itemId, body);
            return localVarResponse.Data;
        }

        /// <summary>
        ///  Creates a custom relationship between an item and another resource within the &#39;data&#39; domain service (folder, item, or version). 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <param name="body">describe the ref to be created</param>
        /// <returns>Task of ApiResponse</returns>
        public async System.Threading.Tasks.Task<ApiResponse<dynamic>> PostItemRelationshipsRefAsyncWithHttpInfo (string projectId, string itemId, CreateRef body)
        {
            // verify the required parameter 'projectId' is set
            if (projectId == null)
                throw new ApiException(400, "Missing required parameter 'projectId' when calling ItemsApi->PostItemRelationshipsRef");
            // verify the required parameter 'itemId' is set
            if (itemId == null)
                throw new ApiException(400, "Missing required parameter 'itemId' when calling ItemsApi->PostItemRelationshipsRef");
            // verify the required parameter 'body' is set
            if (body == null)
                throw new ApiException(400, "Missing required parameter 'body' when calling ItemsApi->PostItemRelationshipsRef");

            var localVarPath = "/data/v1/projects/{project_id}/items/{item_id}/relationships/refs";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new Dictionary<String, String>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/vnd.api+json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/vnd.api+json", 
                "application/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            // set "format" to json by default
            // e.g. /pet/{petId}.{format} becomes /pet/{petId}.json
            localVarPathParams.Add("format", "json");
            if (projectId != null) localVarPathParams.Add("project_id", Configuration.ApiClient.ParameterToString(projectId)); // path parameter
            if (itemId != null) localVarPathParams.Add("item_id", Configuration.ApiClient.ParameterToString(itemId)); // path parameter
            if (body != null && body.GetType() == typeof(byte[])) // http body (model) parameter
            {
                localVarPostBody = body; // byte array
            }
            else if (   body != null
                     && (   body.GetType() == typeof(System.IO.FileStream)
                         || body.GetType() == typeof(System.IO.BinaryReader)
                         || body.GetType() == typeof(System.IO.BufferedStream)
                   	     || body.GetType() == typeof(System.IO.MemoryStream)
                   	     || body.GetType() == typeof(System.IO.StreamReader)
                   	    )
                    )
            {
                localVarPostBody =Configuration.ApiClient.toByteArray(body); // byte array
            }
            else if ( body != null )
            {
                localVarPostBody = Configuration.ApiClient.Serialize(body);
            }

            // authentication (oauth2_access_code) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) await Configuration.ApiClient.CallApiAsync(localVarPath,
                Method.POST, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("PostItemRelationshipsRef", localVarResponse);
                if (exception != null) throw exception;
            }

            
            return new ApiResponse<Object>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                null);
        }

        /// <summary>
        /// Updates the properties of the given item_id object 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <param name="body">describe the ref to be created</param>
        /// <returns>ApiResponse of Object(void)</returns>
        public dynamic PatchItem(string projectId, string itemId, PatchItem body)
        {
            return PatchItemWithHttpInfo(projectId, itemId, body).Data;
        }

        /// <summary>
        /// Updates the properties of the given item_id object
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <param name="body">describe the ref to be created</param>
        /// <returns>ApiResponse of Object(void)</returns>
        public ApiResponse<dynamic> PatchItemWithHttpInfo(string projectId, string itemId, PatchItem body)
        {
            // verify the required parameter 'projectId' is set
            if (projectId == null)
                throw new ApiException(400, "Missing required parameter 'projectId' when calling ItemsApi->PostItemRelationshipsRef");
            // verify the required parameter 'itemId' is set
            if (itemId == null)
                throw new ApiException(400, "Missing required parameter 'itemId' when calling ItemsApi->PostItemRelationshipsRef");
            // verify the required parameter 'body' is set
            if (body == null)
                throw new ApiException(400, "Missing required parameter 'body' when calling ItemsApi->PostItemRelationshipsRef");

            var localVarPath = "/data/v1/projects/{project_id}/items/{item_id}";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new Dictionary<String, String>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/vnd.api+json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/vnd.api+json",
                "application/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            // set "format" to json by default
            // e.g. /pet/{petId}.{format} becomes /pet/{petId}.json
            localVarPathParams.Add("format", "json");
            if (projectId != null) localVarPathParams.Add("project_id", Configuration.ApiClient.ParameterToString(projectId)); // path parameter
            if (itemId != null) localVarPathParams.Add("item_id", Configuration.ApiClient.ParameterToString(itemId)); // path parameter
            if (body != null && body.GetType() == typeof(byte[])) // http body (model) parameter
            {
                localVarPostBody = body; // byte array
            }
            else if (body != null
                     && (body.GetType() == typeof(System.IO.FileStream)
                         || body.GetType() == typeof(System.IO.BinaryReader)
                         || body.GetType() == typeof(System.IO.BufferedStream)
                            || body.GetType() == typeof(System.IO.MemoryStream)
                            || body.GetType() == typeof(System.IO.StreamReader)
                           )
                    )
            {
                localVarPostBody = Configuration.ApiClient.toByteArray(body); // byte array
            }
            else if (body != null)
            {
                localVarPostBody = Configuration.ApiClient.Serialize(body);
            }

            // authentication (oauth2_access_code) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse)Configuration.ApiClient.CallApi(localVarPath,
                Method.PATCH, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int)localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("PatchItem", localVarResponse);
                if (exception != null) throw exception;
            }


            return new ApiResponse<Object>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                /*(JsonApiCollection)*/ Configuration.ApiClient.Deserialize(localVarResponse, typeof(JsonApiCollection)));
        }

        /// <summary>
        ///  Updates the properties of the given item_id object.
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <param name="body">describe the ref to be created</param>
        /// <returns>Task of void</returns>
        public async System.Threading.Tasks.Task<dynamic> PatchItemAsync(string projectId, string itemId, PatchItem body)
        {
            ApiResponse<dynamic> localVarResponse = await PatchItemAsyncWithHttpInfo(projectId, itemId, body);
            return localVarResponse.Data;
        }

        /// <summary>
        /// Updates the properties of the given item_id object.
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="itemId">the &#x60;item id&#x60;</param>
        /// <param name="body">describe the ref to be created</param>
        /// <returns>Task of ApiResponse</returns>
        public async System.Threading.Tasks.Task<ApiResponse<dynamic>> PatchItemAsyncWithHttpInfo(string projectId, string itemId, PatchItem body)
        {
            // verify the required parameter 'projectId' is set
            if (projectId == null)
                throw new ApiException(400, "Missing required parameter 'projectId' when calling ItemsApi->PostItemRelationshipsRef");
            // verify the required parameter 'itemId' is set
            if (itemId == null)
                throw new ApiException(400, "Missing required parameter 'itemId' when calling ItemsApi->PostItemRelationshipsRef");
            // verify the required parameter 'body' is set
            if (body == null)
                throw new ApiException(400, "Missing required parameter 'body' when calling ItemsApi->PostItemRelationshipsRef");

            var localVarPath = "/data/v1/projects/{project_id}/items/{item_id}";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new Dictionary<String, String>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/vnd.api+json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/vnd.api+json",
                "application/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            // set "format" to json by default
            // e.g. /pet/{petId}.{format} becomes /pet/{petId}.json
            localVarPathParams.Add("format", "json");
            if (projectId != null) localVarPathParams.Add("project_id", Configuration.ApiClient.ParameterToString(projectId)); // path parameter
            if (itemId != null) localVarPathParams.Add("item_id", Configuration.ApiClient.ParameterToString(itemId)); // path parameter
            if (body != null && body.GetType() == typeof(byte[])) // http body (model) parameter
            {
                localVarPostBody = body; // byte array
            }
            else if (body != null
                     && (body.GetType() == typeof(System.IO.FileStream)
                         || body.GetType() == typeof(System.IO.BinaryReader)
                         || body.GetType() == typeof(System.IO.BufferedStream)
                            || body.GetType() == typeof(System.IO.MemoryStream)
                            || body.GetType() == typeof(System.IO.StreamReader)
                           )
                    )
            {
                localVarPostBody = Configuration.ApiClient.toByteArray(body); // byte array
            }
            else if (body != null)
            {
                localVarPostBody = Configuration.ApiClient.Serialize(body);
            }

            // authentication (oauth2_access_code) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse)await Configuration.ApiClient.CallApiAsync(localVarPath,
                Method.PATCH, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int)localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("PatchItem", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse<Object>(localVarStatusCode,
               localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
               null);

            /*return new ApiResponse<Object>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                (JsonApiCollection) Configuration.ApiClient.Deserialize(localVarResponse, typeof(JsonApiCollection)));*/
        }

    }
}
