/* 
 * Forge SDK
 *
 * The Forge Platform contains an expanding collection of web service components that can be used with Autodesk cloud-based products or your own technologies. Take advantage of Autodesk’s expertise in design and engineering.
 *
 * OpenAPI spec version: 0.1.0
 * Contact: forge.help@autodesk.com
 * Generated by: https://github.com/swagger-api/swagger-codegen.git
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

using System;
using System.Linq;
using System.IO;
using System.Text;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;

namespace Autodesk.Forge.Model
{
    /// <summary>
    /// FolderAttributes
    /// </summary>
    [DataContract]
    public partial class FolderAttributes :  IEquatable<FolderAttributes>
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="FolderAttributes" /> class.
        /// </summary>
        [JsonConstructorAttribute]
        protected FolderAttributes() { }
        /// <summary>
        /// Initializes a new instance of the <see cref="FolderAttributes" /> class.
        /// </summary>
        /// <param name="Name">filename used when synced to local disk (required).</param>
        /// <param name="DisplayName">displayable name of the folder (required).</param>
        /// <param name="ObjectCount">number of contained sub-folders and items (required).</param>
        /// <param name="Extension">Extension (required).</param>
        public FolderAttributes(string Name = null, string DisplayName = null, int? ObjectCount = null, BaseAttributesExtensionObject Extension = null)
        {
            // to ensure "Name" is required (not null)
            if (Name == null)
            {
                throw new InvalidDataException("Name is a required property for FolderAttributes and cannot be null");
            }
            else
            {
                this.Name = Name;
            }
            // to ensure "DisplayName" is required (not null)
            if (DisplayName == null)
            {
                throw new InvalidDataException("DisplayName is a required property for FolderAttributes and cannot be null");
            }
            else
            {
                this.DisplayName = DisplayName;
            }
            // to ensure "ObjectCount" is required (not null)
            if (ObjectCount == null)
            {
                throw new InvalidDataException("ObjectCount is a required property for FolderAttributes and cannot be null");
            }
            else
            {
                this.ObjectCount = ObjectCount;
            }
            // to ensure "Extension" is required (not null)
            if (Extension == null)
            {
                throw new InvalidDataException("Extension is a required property for FolderAttributes and cannot be null");
            }
            else
            {
                this.Extension = Extension;
            }
        }
        
        /// <summary>
        /// filename used when synced to local disk
        /// </summary>
        /// <value>filename used when synced to local disk</value>
        [DataMember(Name="name", EmitDefaultValue=false)]
        public string Name { get; set; }
        /// <summary>
        /// displayable name of the folder
        /// </summary>
        /// <value>displayable name of the folder</value>
        [DataMember(Name="displayName", EmitDefaultValue=false)]
        public string DisplayName { get; set; }
        /// <summary>
        /// number of contained sub-folders and items
        /// </summary>
        /// <value>number of contained sub-folders and items</value>
        [DataMember(Name="objectCount", EmitDefaultValue=false)]
        public int? ObjectCount { get; set; }
        /// <summary>
        /// Gets or Sets Extension
        /// </summary>
        [DataMember(Name="extension", EmitDefaultValue=false)]
        public BaseAttributesExtensionObject Extension { get; set; }
        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            var sb = new StringBuilder();
            sb.Append("class FolderAttributes {\n");
            sb.Append("  Name: ").Append(Name).Append("\n");
            sb.Append("  DisplayName: ").Append(DisplayName).Append("\n");
            sb.Append("  ObjectCount: ").Append(ObjectCount).Append("\n");
            sb.Append("  Extension: ").Append(Extension).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }
  
        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public string ToJson()
        {
            return JsonConvert.SerializeObject(this, Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="obj">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object obj)
        {
            // credit: http://stackoverflow.com/a/10454552/677735
            return this.Equals(obj as FolderAttributes);
        }

        /// <summary>
        /// Returns true if FolderAttributes instances are equal
        /// </summary>
        /// <param name="other">Instance of FolderAttributes to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(FolderAttributes other)
        {
            // credit: http://stackoverflow.com/a/10454552/677735
            if (other == null)
                return false;

            return 
                (
                    this.Name == other.Name ||
                    this.Name != null &&
                    this.Name.Equals(other.Name)
                ) && 
                (
                    this.DisplayName == other.DisplayName ||
                    this.DisplayName != null &&
                    this.DisplayName.Equals(other.DisplayName)
                ) && 
                (
                    this.ObjectCount == other.ObjectCount ||
                    this.ObjectCount != null &&
                    this.ObjectCount.Equals(other.ObjectCount)
                ) && 
                (
                    this.Extension == other.Extension ||
                    this.Extension != null &&
                    this.Extension.Equals(other.Extension)
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            // credit: http://stackoverflow.com/a/263416/677735
            unchecked // Overflow is fine, just wrap
            {
                int hash = 41;
                // Suitable nullity checks etc, of course :)
                if (this.Name != null)
                    hash = hash * 59 + this.Name.GetHashCode();
                if (this.DisplayName != null)
                    hash = hash * 59 + this.DisplayName.GetHashCode();
                if (this.ObjectCount != null)
                    hash = hash * 59 + this.ObjectCount.GetHashCode();
                if (this.Extension != null)
                    hash = hash * 59 + this.Extension.GetHashCode();
                return hash;
            }
        }
    }

}

