/* 
 * Forge SDK
 *
 * The Forge Platform contains an expanding collection of web service components that can be used with Autodesk cloud-based products or your own technologies. Take advantage of Autodesk’s expertise in design and engineering.
 *
 * OpenAPI spec version: 0.1.0
 * Contact: forge.help@autodesk.com
 * Generated by: https://github.com/swagger-api/swagger-codegen.git
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

using System;
using System.Linq;
using System.IO;
using System.Text;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;

namespace Autodesk.Forge.Model
{
    /// <summary>
    /// Bucket Payload Body Structure
    /// </summary>
    [DataContract]
    public partial class PostBucketsPayload :  IEquatable<PostBucketsPayload>
    {
        /// <summary>
        /// [Data retention policy](https://developer.autodesk.com/en/docs/data/v2/overview/retention-policy/)  Acceptable values: `transient`, `temporary` or `persistent` 
        /// </summary>
        /// <value>[Data retention policy](https://developer.autodesk.com/en/docs/data/v2/overview/retention-policy/)  Acceptable values: `transient`, `temporary` or `persistent` </value>
        [JsonConverter(typeof(StringEnumConverter))]
        public enum PolicyKeyEnum
        {
            
            /// <summary>
            /// Enum Transient for "transient"
            /// </summary>
            [EnumMember(Value = "transient")]
            Transient,
            
            /// <summary>
            /// Enum Temporary for "temporary"
            /// </summary>
            [EnumMember(Value = "temporary")]
            Temporary,
            
            /// <summary>
            /// Enum Persistent for "persistent"
            /// </summary>
            [EnumMember(Value = "persistent")]
            Persistent
        }

        /// <summary>
        /// [Data retention policy](https://developer.autodesk.com/en/docs/data/v2/overview/retention-policy/)  Acceptable values: `transient`, `temporary` or `persistent` 
        /// </summary>
        /// <value>[Data retention policy](https://developer.autodesk.com/en/docs/data/v2/overview/retention-policy/)  Acceptable values: `transient`, `temporary` or `persistent` </value>
        [DataMember(Name="policyKey", EmitDefaultValue=false)]
        public PolicyKeyEnum? PolicyKey { get; set; }
        /// <summary>
        /// Initializes a new instance of the <see cref="PostBucketsPayload" /> class.
        /// </summary>
        [JsonConstructorAttribute]
        protected PostBucketsPayload() { }
        /// <summary>
        /// Initializes a new instance of the <see cref="PostBucketsPayload" /> class.
        /// </summary>
        /// <param name="BucketKey">Bucket key (required).</param>
        /// <param name="Allow">.</param>
        /// <param name="PolicyKey">[Data retention policy](https://developer.autodesk.com/en/docs/data/v2/overview/retention-policy/)  Acceptable values: &#x60;transient&#x60;, &#x60;temporary&#x60; or &#x60;persistent&#x60;  (required).</param>
        public PostBucketsPayload(string BucketKey = null, List<PostBucketsPayloadAllow> Allow = null, PolicyKeyEnum? PolicyKey = null)
        {
            // to ensure "BucketKey" is required (not null)
            if (BucketKey == null)
            {
                throw new InvalidDataException("BucketKey is a required property for PostBucketsPayload and cannot be null");
            }
            else
            {
                this.BucketKey = BucketKey;
            }
            // to ensure "PolicyKey" is required (not null)
            if (PolicyKey == null)
            {
                throw new InvalidDataException("PolicyKey is a required property for PostBucketsPayload and cannot be null");
            }
            else
            {
                this.PolicyKey = PolicyKey;
            }
            this.Allow = Allow;
        }
        
        /// <summary>
        /// Bucket key
        /// </summary>
        /// <value>Bucket key</value>
        [DataMember(Name="bucketKey", EmitDefaultValue=false)]
        public string BucketKey { get; set; }
        /// <summary>
        /// 
        /// </summary>
        /// <value></value>
        [DataMember(Name="allow", EmitDefaultValue=false)]
        public List<PostBucketsPayloadAllow> Allow { get; set; }
        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            var sb = new StringBuilder();
            sb.Append("class PostBucketsPayload {\n");
            sb.Append("  BucketKey: ").Append(BucketKey).Append("\n");
            sb.Append("  Allow: ").Append(Allow).Append("\n");
            sb.Append("  PolicyKey: ").Append(PolicyKey).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }
  
        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public string ToJson()
        {
            return JsonConvert.SerializeObject(this, Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="obj">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object obj)
        {
            // credit: http://stackoverflow.com/a/10454552/677735
            return this.Equals(obj as PostBucketsPayload);
        }

        /// <summary>
        /// Returns true if PostBucketsPayload instances are equal
        /// </summary>
        /// <param name="other">Instance of PostBucketsPayload to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(PostBucketsPayload other)
        {
            // credit: http://stackoverflow.com/a/10454552/677735
            if (other == null)
                return false;

            return 
                (
                    this.BucketKey == other.BucketKey ||
                    this.BucketKey != null &&
                    this.BucketKey.Equals(other.BucketKey)
                ) && 
                (
                    this.Allow == other.Allow ||
                    this.Allow != null &&
                    this.Allow.SequenceEqual(other.Allow)
                ) && 
                (
                    this.PolicyKey == other.PolicyKey ||
                    this.PolicyKey != null &&
                    this.PolicyKey.Equals(other.PolicyKey)
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            // credit: http://stackoverflow.com/a/263416/677735
            unchecked // Overflow is fine, just wrap
            {
                int hash = 41;
                // Suitable nullity checks etc, of course :)
                if (this.BucketKey != null)
                    hash = hash * 59 + this.BucketKey.GetHashCode();
                if (this.Allow != null)
                    hash = hash * 59 + this.Allow.GetHashCode();
                if (this.PolicyKey != null)
                    hash = hash * 59 + this.PolicyKey.GetHashCode();
                return hash;
            }
        }
    }

}

